# Copyright(c) Parag Bajaria
#Licensed under GPL

import imp
import sys
import getopt
import glob

filesPatterns = []
reports = []

fileGmatData = {}

class Question:
    def __str__(self):
        return "num=%d:correct=%d:type0=%s:type1=%s:timeInSecs=%d:comment=%s"%\
                (self.num,self.correct,self.type0,self.type1,self.timeInSecs,self.comment) 

class GmatData:
    def printData(self):
        print "section=%s:section comment=%s:date=%s"%(self.section,self.sectionComment,self.date)
        for quest in self.questions:
            print quest

def readFileData(fileName):
    fileData = open(fileName).readlines()
    if len(fileData) == 0:
       return fileData
    fileDataTemp = []
    for i in range(0,len(fileData)-1):
        if len(fileData[i])==1 and (fileData[i] == "" or fileData[i]=="\n"):
            continue
        if fileData[i][0] == '#':
            continue
        fileDataTemp.append(fileData[i])
    return fileDataTemp

def parseTime(timeStr):
    if timeStr=="":
        return 0
    temp = timeStr.split(".")
    return 60*int(temp[0]) + int(temp[1])

def parseFile(fileName):                              
    fileData = readFileData(fileName)
    if len(fileData) == 0:
        print "Ignoring empty file %s"%fileName
        return
    try:
        sectionInfo = fileData[0][:-1].split(":")
        if len(sectionInfo) != 3:
            raise "Bad Section information in file %s"%fileName
        gmatData = GmatData()
        gmatData.questions = []
        gmatData.section = sectionInfo[0].split("=")[1].strip()
        gmatData.sectionComment = sectionInfo[1].split("=")[1].strip()
        gmatData.date = sectionInfo[2].split("=")[1].strip()

        for i in range(1,len(fileData)):
            questInfo = fileData[i][:-1].split(":")
            if questInfo[0]=="":
                continue
            if len(questInfo) != 6:
                print "Ignoring line %d of file %s, bad format.Expecting 6 values seperated by \":\", instead got %d values."%(i,fileName,len(questInfo))
                continue
            quest = Question()
            quest.num = int(questInfo[0].split("=")[1].strip())
            correct = int(questInfo[1].split("=")[1].strip())
            if correct:
                quest.correct = True
            else:
                quest.correct = False
            quest.type0 = questInfo[2].split("=")[1].strip().lower()
            quest.type1 = questInfo[3].split("=")[1].strip().split(",")
            for i in range(0,len(quest.type1)):
                quest.type1[i] = quest.type1[i].lower()
                
            quest.timeInSecs = parseTime(questInfo[4].split("=")[1].strip())
            quest.comment = questInfo[5].split("=")[1].strip()

            gmatData.questions.append(quest)

        fileGmatData[fileName] = gmatData
        #print "!!! "+fileName
        #gmatData.printData()
    except:
        print "Invalid file format"
        raise
        


def loadDataFiles():
    allFileNames = []
    for filePattern in filesPatterns:
        fileNames = glob.glob(filePattern)
        if len(fileNames) > 0:
            for fileName in fileNames:
                allFileNames.append(fileName)
    if len(allFileNames):
        for fileName in allFileNames:
            parseFile(fileName)
    else:
        print "No files match"
        sys.exit(2)

def loadReports():
    for report in reports:
        try:
            fndFile,fndName,fndDesc = imp.find_module(report)
            #imp.load_module(report)
        except ImportError:
            raise

        imp.load_module(report,fndFile,fndName,fndDesc)
        func = getattr(sys.modules[report], "generateReport")
        apply(func,[fileGmatData])


def printGmatData():
    #print fileGmatData
    for fileName in fileGmatData.keys():
        print fileName + ":"
        fileGmatData[fileName].printData()




def Usage():
    print '''
    Options:
        -h display this help messsage
        -f data file/files
        -r report name
    '''

def doMain():
    try:
        optlist, list  = getopt.getopt(sys.argv[1:],"h:f:r:")
    except getopt.GetoptError:
        Usage();
        sys.exit(2)

    if len(optlist) == 0:
        Usage();
        sys.exit(2)
    
    for opt in optlist:
        if opt[0] == '-h':
            Usage()
            break
        if opt[0] == '-f':
            filesPatterns.append(opt[1])
        if opt[0] == '-r':
            reports.append(opt[1])
    if len(filesPatterns) == 0 :
        print "No file/files specified"
        Usage()
        sys.exit(2)
    if len(reports) == 0 :
        print "No report/reports specified"
        Usage()
        sys.exit(2)

    loadDataFiles() 
    #printGmatData()
    loadReports()  

if __name__ == '__main__':
    doMain()

def test():
    modName="quant_correct"
    fndFile,fndName,fndDesc = imp.find_module(modName)
    print fndName
    imp.load_module(modName,fndFile,fndName,fndDesc)
    func = getattr(sys.modules[modName], "generateReport")
    apply(func,[])
